/*
	Klient printserveru

	Autor: Martin Bruchanov (brucham@feld.cvut.cz)

	36DSY 2005/2006

   */

#include "com.h"


/*******************************
 *                             *
 *        F u n k c e          *
 *                             *
 *                             *
 *******************************/

void print_usage(void){
	fprintf(stderr,
		"Usage:\n" \
		"\t -p port \n"\
		"\t -h host (IP address)\n"\
		"\t -f filename (file to print)\n"\
		"\t -s job (stop this job)\n"\
		"\t -c job (cancel)\n"\
		"\t -r job (restart stopped job)\n"\
		"\t -n priority\n"\
		"\t -d (display current print jobs)\n"\
		"\n"
		);
	return;
}

int get_options(int argc, char *argv[], parametry_spojeni *spoj_info){
	int i;
	
	spoj_info->port = 0;
	
	for(i = 1; i < argc; i++){
		if(strcmp(argv[i], "-p") == 0){
		/* Port */
			spoj_info->port = (unsigned short int) atoi(argv[i+1]);
		}
		if(strcmp(argv[i], "-h") == 0){			
		/* Server Host IP */
			strncpy(spoj_info->ip, argv[i+1], 16);
		}
		if(strcmp(argv[i], "-f") == 0){			
		/* File name */
			strncpy(spoj_info->filename, argv[i+1], 255);
			spoj_info->todo = PRINT; /* akce tisk */
		}
		if(strcmp(argv[i], "-s") == 0){			
			spoj_info->job = (unsigned int)	atoi(argv[i+1]);	
			spoj_info->todo = STOP;	
		}	
		if(strcmp(argv[i], "-c") == 0){			
			spoj_info->job = (unsigned int)	atoi(argv[i+1]);	
			spoj_info->todo = CANCEL;	
		}	
		if(strcmp(argv[i], "-r") == 0){			
			spoj_info->job = (unsigned int)	atoi(argv[i+1]);	
			spoj_info->todo = START;	
		}	
		if(strcmp(argv[i], "-d") == 0){			
			spoj_info->todo = DISPLAY;	
		}	
	}
	
	/* User ID a login pro nastaveni */
	spoj_info->userid = (int) geteuid();
	cuserid(spoj_info->username);
	return 0;
}



int check_options(const parametry_spojeni *spoj_info){
	if(!spoj_info->port || !spoj_info->ip){
		print_usage();
		return(-1);
	}

	/* zjisteni existence souboru */
	if(spoj_info->filename){
		if( access(spoj_info->filename, R_OK|F_OK) < 0 ){
			fprintf(stderr, "filename '%s': %s\n", spoj_info->filename, strerror(errno));
			return(-1);
		}
	}
	return 1;
}

int create_socket(const parametry_spojeni *spoj_info){
	int sock;
	struct sockaddr_in adresa;
	socketset(&adresa, spoj_info);
	if((sock = connectto(&adresa)) == -1) return -1;
	return(sock);
}


int sendfile(int sock, const parametry_spojeni *spoj_info){
	int fd;
/*	struct sockaddr_in adresa;*/
	ssize_t count;
	char buffer[BLOCK_SIZE];
	int size = 0;
	
	if((fd = openfile((char *) spoj_info->filename, O_RDONLY)) == -1) return -1;

	/* socketset(&adresa, spoj_info);
	if((sock = connectto(&adresa)) == -1) return -1;
 */
	while((count = read(fd, buffer, sizeof(buffer))) > 0){
		if(send(sock, buffer, count, 0) == -1){
			fprintf(stderr, "send: %s\n", strerror(errno));
			return(-1);
		}
		size += count; 
	}
	return size; 
}

int send_info(int sock, const parametry_spojeni *spoj_info){
/*
	struct sockaddr_in adresa;
	socketset(&adresa, spoj_info);
	if((sock = connectto(&adresa)) == -1) return -1;
*/
	
	if(send(sock, spoj_info, sizeof(parametry_spojeni), 0) == -1){
	     fprintf(stderr, "send_info: %s\n", strerror(errno));
	     return(-1);
	 }
	return(0);
}

/*******************************
 *                             *
 *   P r i n t  c l i e n t    *
 *                             *
 *                             *
 *******************************/

int main(int argc, char *argv[]){
	int socket;
	parametry_spojeni client_data;
/*	struct sockaddr_in adresa;*/
	
	memset(&client_data, 0, sizeof(client_data));

	/* Nacteni parametru */
	get_options(argc, argv, &client_data);
	if(check_options(&client_data) < 0) exit(-1);
/*	print_options(&client_data); */

	if(!(socket = create_socket(&client_data))) exit(-1);
	
	switch(client_data.todo){
		case PRINT: /* Odeslani souboru */
			printf("Posilam soubor: %s\n", client_data.filename); 
			send_info(socket, &client_data);
			sendfile(socket, &client_data);
			/* Cekani na odpoved */
			break;
		case CANCEL: 	
		case STOP:  	
		case START: 	
		case PRIORITY: 	
		case DISPLAY: 
			printf("Zatim neni implementovano!\n");
			break;
	}

	return 0;
}
