/*
   Print server
   Autor: Martin Bruchanov
   36DSY 2005/2006

   */

#include "com.h"

#include <time.h>
#include <signal.h>


#define LISTEN_PORT 5556
#define PRINT_CHAR  10000 /* pocet mikrosekund pro vytisk jednoho znaku */

int server_settings(struct sockaddr_in *adresa);

/************************************************************************/
/* Osetreni ukonceni programu pri CTRL+C a prijeti 
      signalu SIGTERM */

sig_atomic_t QUIT = 0; /* if 1 KONEC */

void ukonceni(int prom){
		QUIT++;	
}
/************************************************************************/


int server_socketset(struct sockaddr_in *adresa, unsigned short int port){
	/* Nastaveni parametru spojeni */
	memset(adresa, 0, sizeof(struct sockaddr_in));
	adresa->sin_family = AF_INET;
	adresa->sin_port = htons(port);
	adresa->sin_addr.s_addr = (unsigned long int) inet_addr("0.0.0.0");
		/* INADDR_ANY; */
	return 0;	
}

void create_unique_name(parametry_spojeni *info_packet){
	/* Prida na konec souboru cas pozadavku */
	char filename_tail[12]; /* pr. -1136122467 */
	filename_tail[0] = '-';
	filename_tail[11] = 0;

	
	sprintf(&filename_tail[1], "%lu", (unsigned long int) time(NULL));
	strcat(info_packet->filename, filename_tail);

#if DEBUG
	printf("Novy soubor: %s\n", info_packet->filename);
#endif
	return;
}


int recvfile(int socket, parametry_spojeni *spoj_info){
	int fd;
/*	struct sockaddr_in adresa; */
	ssize_t count;
	char buffer[BLOCK_SIZE];
	int size = 0;

	create_unique_name(spoj_info); /* unikatni pojmenovani souboru */

	creat(spoj_info->filename, S_IRUSR|S_IWUSR);
	if((fd = openfile((char *) spoj_info->filename, O_WRONLY)) == -1) return -1;

	/* server_socketset(&adresa, LISTEN_PORT);
	if((sock = server_settings(&adresa)) == -1) return -1;
    */

#if DEBUG
	printf("Prijimam data ze socketu cislo %d\n", socket);
#endif
	
	while((count = recv(socket, buffer, sizeof(BLOCK_SIZE), MSG_NOSIGNAL )) > 0){
    /* MSG_NOSIGNAL -- neni posilan signal SIGPIPE, kdyz druha strana
	   zrusi spojeni */
		if(write(fd, buffer, count) == -1){
			fprintf(stderr, "write: %s\n", strerror(errno));
			return(-1);
		}
		size += count; 
	}
	close(fd);
	return size; 
}

int server_settings(struct sockaddr_in *adresa){
	int sd;
	int option = 1;

	if((sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1)
	{
		fprintf(stderr, "socket: %s\n", strerror(errno));
		return -1;
	}
	if (setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &option, sizeof(option)) == -1)
    {
        fprintf(stderr, "setsockopt: %s\n", strerror(errno));
        return -1;
    }
	if((bind(sd, (struct sockaddr *) adresa, sizeof(struct sockaddr_in))) == -1)
	{ 
		fprintf(stderr, "bind: %s.\n", strerror(errno));	
		return -1;
	}
	if(listen(sd, SOMAXCONN) == -1){ /* bit/socket.h SOMAXCONN = 128 */
		fprintf(stderr, "bind: %s.\n", strerror(errno));	
		return -1;
	}

	return sd;
}

int recv_info(int socket, parametry_spojeni *info_packet){
	if(recv(socket, info_packet, sizeof(parametry_spojeni), MSG_NOSIGNAL) != sizeof(parametry_spojeni)){
		fprintf(stderr, "recv_info: info packet not received.\n");
		return(-1);
	}
	return(0); 
}

int print_file(char *filename){
	int fp;
	char znak;

	if((fp = openfile(filename, O_RDONLY)) == -1){
		fprintf(stderr, "print_file: Unable to open file %s\n", filename );
		return(-1);
	}

	while(read(fp, &znak, 1)>0){
		usleep(PRINT_CHAR);
		write(1, &znak, 1);
	}

	/* Konec tisku */
	printf("- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - \n");
	remove(filename);
	
	close(fp);
	return(0);
}



/*******************************
 *                             *
 *   P r i n t  s e r v e r    *
 *                             *
 *                             *
 *******************************/

int main(int argc, char *argv[]){
	struct sockaddr_in adresa;
	int sd, csocket;
	parametry_spojeni info_packet; /* Informace o prijatem packetu */
	/* podprocesy */
	pid_t prijem;
	pid_t tisk; 
	/* roura pro predavani */
	int kanal[2]; /* kanal[0] cteni, kanal[1] zapis */


	/* Obsluha signalu SIGINT a SIGTERM */
	signal(SIGINT, &ukonceni);
	signal(SIGTERM, &ukonceni);
	
	
#if 0

	/* Ukonceni pomoci signalu SIGINT */
	struct sigaction akce;

	/* Obsluha signalu SIGINT a SIGINT */
	memset (&akce, 0, sizeof (akce));
	akce.sa_handler = (void *) &ukonceni;
	sigaction(SIGINT, &akce, NULL); /* CTRL + C*/
	sigaction(SIGTERM, &akce, NULL); /* kill */

#endif 
	
	if(pipe(kanal) == -1){
		fprintf(stderr, "pipe: %s\n", strerror(errno));
		exit(-1);
	}
	
	
	tisk = fork();
	/*** Tiskovy proces *************************************************/
	if(!tisk){
		char filename[256];
		close(kanal[1]);
		printf("Tiskovy podproces nahozen!\n");
		for(;;){
			int count = 0;
			if((count = read(kanal[0], filename, sizeof(filename))) == sizeof(filename)){
				printf("Tisknu soubor (%s)!\n", filename);
				print_file(filename);
			}

			if(QUIT){
				printf("Ukoncen tiskovy proces (%u)!\n", getpid());
				exit(0);
			}
		}	
	}
	/*******************************************************************/
	
	server_socketset(&adresa, LISTEN_PORT);
	if((sd = server_settings(&adresa)) == -1){
		fprintf(stderr, "Server settings failed!\n");
		exit(-1);
	}

	
	/*******************************************************************/
for(;;){ /* Nekonecna smycka prijmu */
	if(QUIT) break;

	if((csocket = accept(sd, NULL, 0)) == -1){
		fprintf(stderr, "accept: %s.\n", strerror(errno));	
		return -1;
	}

	recv_info(csocket, &info_packet); /* Informace odkud prisel pozadavek */
#if DEBUG
	print_options(&info_packet);
#endif
	
	prijem = fork();
	/*** Proces prijmu souboru *****************************************/
	if(!prijem)	
	{
		close(sd);
		recvfile(csocket, &info_packet);
		close(csocket);
		/* Odeslu to roury pozadavek k tisku */
		
		printf("\n[ \033[01;36mPozadavek tisku: %s\033[m\033(B]\n", info_packet.filename);
		
		write(kanal[1], info_packet.filename, sizeof(info_packet.filename));
		exit(0);
	}
	/*******************************************************************/

	close(kanal[0]);
	close(csocket);

	}	
	waitpid(tisk, NULL, 0);
	waitpid(prijem, NULL, 0);
	close(kanal[1]);
	return 0;
}
